# Basic version of Zarr storage interface

# Doesn't support deleting, or changing data already written.

using ArgCheck
using ZipArchives


abstract type AbstractWriter end

"""
Call to finish writing.
"""
function closewriter(::AbstractWriter)
end


struct DirectoryWriter <: AbstractWriter
    path::String
    function DirectoryWriter(dir)
        mkpath(dir)
        new(abspath(dir))
    end
end

"""
Add a key and value to the store.
"""
function write_key(d::DirectoryWriter, key::AbstractString, data)::Nothing
    @assert !endswith(key, "/")
    filename = joinpath([d.path; split(key,"/")])
    mkpath(dirname(filename))
    open(filename, "w") do f
        write(f, data)
    end
    nothing
end

"""
Write to an in memory zipfile, that gets saved to disk on close.
This writer will overwrite any existing file at `path`
"""
struct BufferedZipWriter <: AbstractWriter
    zipfile::ZipWriter{IOBuffer}
    path::String
    iobuffer::IOBuffer
    function BufferedZipWriter(path)
        @argcheck !isdir(path)
        @argcheck !isdirpath(path)
        mkpath(dirname(path))
        iobuffer = IOBuffer()
        zipfile = ZipWriter(iobuffer)
        new(zipfile, abspath(path), iobuffer)
    end
end

function write_key(d::BufferedZipWriter, key::AbstractString, data)::Nothing
    zip_writefile(d.zipfile, key, data)
    nothing
end

function closewriter(d::BufferedZipWriter)
    close(d.zipfile)
    open(d.path, "w") do f
        write(f, take!(d.iobuffer))
    end
end